# generated by datamodel-codegen:
#   filename:  aqt_public.yml

from __future__ import annotations

from enum import Enum
from typing import Dict, List, Literal, Optional, Union
from uuid import UUID

from pydantic import BaseModel, Extra, Field
from typing_extensions import Annotated


class GateR(BaseModel):
    """
    A single-qubit rotation of angle θ around axis φ in the equatorial plane
    of the Bloch sphere.

    Angles are expressed in units of π.
    """

    class Config:
        extra = Extra.forbid
        allow_mutation = False

    operation: Annotated[Literal["R"], Field(title="Operation")]
    phi: Annotated[float, Field(ge=0.0, le=2.0, title="Phi")]
    qubit: Annotated[int, Field(ge=0, title="Qubit")]
    theta: Annotated[float, Field(ge=0.0, le=1.0, title="Theta")]


class Qubit(BaseModel):
    class Config:
        allow_mutation = False

    __root__: Annotated[int, Field(ge=0)]


class GateRXX(BaseModel):
    """
    A parametric 2-qubits X⊗X gate with angle θ.

    The angle is expressed in units of π. The gate is maximally entangling
    for θ=0.5 (π/2).
    """

    class Config:
        extra = Extra.forbid
        allow_mutation = False

    operation: Annotated[Literal["RXX"], Field(title="Operation")]
    qubits: Annotated[
        List[Qubit], Field(max_items=2, min_items=2, title="Qubits", unique_items=True)
    ]
    theta: Annotated[float, Field(ge=0.0, le=0.5, title="Theta")]


class GateRZ(BaseModel):
    """
    A single-qubit rotation of angle φ around the Z axis of the Bloch sphere.
    """

    class Config:
        extra = Extra.forbid
        allow_mutation = False

    operation: Annotated[Literal["RZ"], Field(title="Operation")]
    phi: Annotated[float, Field(title="Phi")]
    qubit: Annotated[int, Field(ge=0, title="Qubit")]


class JobUser(BaseModel):
    """
    Abstract job that can run on a computing resource.
    """

    class Config:
        allow_mutation = False

    job_id: Annotated[UUID, Field(title="Job Id")]
    """
    Id that uniquely identifies the job. This is used to request results.
    """
    job_type: Annotated[Literal["quantum_circuit"], Field(title="Job Type")] = "quantum_circuit"
    label: Annotated[Optional[str], Field(title="Label")] = None
    resource_id: Annotated[str, Field(title="Resource Id")] = ""
    workspace_id: Annotated[str, Field(title="Workspace Id")] = ""


class Measure(BaseModel):
    """
    Measurement operation.

    The MEASURE operation instructs the resource
    to perform a projective measurement of all qubits.
    """

    class Config:
        extra = Extra.forbid
        allow_mutation = False

    operation: Annotated[Literal["MEASURE"], Field(title="Operation")]


class OperationModel(BaseModel):
    class Config:
        allow_mutation = False

    __root__: Annotated[
        Union[GateRZ, GateR, GateRXX, Measure],
        Field(discriminator="operation", title="OperationModel"),
    ]
    """
    Model for the items in a Circuit.

    This extra wrapper is introduced to leverage the pydantic
    tagged-union parser.
    """


class RRCancelled(BaseModel):
    class Config:
        allow_mutation = False

    status: Annotated[Literal["cancelled"], Field(title="Status")] = "cancelled"


class RRError(BaseModel):
    class Config:
        allow_mutation = False

    message: Annotated[str, Field(title="Message")]
    status: Annotated[Literal["error"], Field(title="Status")] = "error"


class ResultItem(BaseModel):
    class Config:
        allow_mutation = False

    __root__: Annotated[int, Field(ge=0, le=1)]


class RRFinished(BaseModel):
    """
    Contains the measurement data of a finished circuit.
    """

    class Config:
        allow_mutation = False

    result: Annotated[Dict[str, List[List[ResultItem]]], Field(title="Result")]
    status: Annotated[Literal["finished"], Field(title="Status")] = "finished"


class RROngoing(BaseModel):
    class Config:
        allow_mutation = False

    finished_count: Annotated[int, Field(ge=0, title="Finished Count")]
    status: Annotated[Literal["ongoing"], Field(title="Status")] = "ongoing"


class RRQueued(BaseModel):
    class Config:
        allow_mutation = False

    status: Annotated[Literal["queued"], Field(title="Status")] = "queued"


class Type(Enum):
    simulator = "simulator"
    device = "device"


class Resource(BaseModel):
    class Config:
        allow_mutation = False

    id: Annotated[str, Field(title="Id")]
    name: Annotated[str, Field(title="Name")]
    type: Annotated[Type, Field(title="Type")]


class UnknownJob(BaseModel):
    class Config:
        allow_mutation = False

    job_id: Annotated[UUID, Field(title="Job Id")]
    message: Annotated[Literal["unknown job_id"], Field(title="Message")] = "unknown job_id"


class ValidationError(BaseModel):
    class Config:
        allow_mutation = False

    loc: Annotated[List[Union[str, int]], Field(title="Location")]
    msg: Annotated[str, Field(title="Message")]
    type: Annotated[str, Field(title="Error Type")]


class Workspace(BaseModel):
    class Config:
        allow_mutation = False

    id: Annotated[str, Field(title="Id")]
    resources: Annotated[List[Resource], Field(title="Resources")]


class Circuit(BaseModel):
    """
    Json encoding of a quantum circuit.
    """

    class Config:
        allow_mutation = False

    __root__: Annotated[
        List[OperationModel],
        Field(
            example=[
                {"operation": "RZ", "phi": 0.5, "qubit": 0},
                {"operation": "R", "phi": 0.25, "qubit": 1, "theta": 0.5},
                {"operation": "RXX", "qubits": [0, 1], "theta": 0.5},
                {"operation": "MEASURE"},
            ],
            max_items=10000,
            min_items=1,
            title="Circuit",
        ),
    ]
    """
    Json encoding of a quantum circuit.
    """


class HTTPValidationError(BaseModel):
    class Config:
        allow_mutation = False

    detail: Annotated[Optional[List[ValidationError]], Field(title="Detail")] = None


class JobResponseRRCancelled(BaseModel):
    """
    This class contains the data a uses is receiving at the "/result" endpoint.
    """

    class Config:
        allow_mutation = False

    job: JobUser
    response: RRCancelled


class JobResponseRRError(BaseModel):
    """
    This class contains the data a uses is receiving at the "/result" endpoint.
    """

    class Config:
        allow_mutation = False

    job: JobUser
    response: RRError


class JobResponseRRFinished(BaseModel):
    """
    This class contains the data a uses is receiving at the "/result" endpoint.
    """

    class Config:
        allow_mutation = False

    job: JobUser
    response: RRFinished


class JobResponseRROngoing(BaseModel):
    """
    This class contains the data a uses is receiving at the "/result" endpoint.
    """

    class Config:
        allow_mutation = False

    job: JobUser
    response: RROngoing


class JobResponseRRQueued(BaseModel):
    """
    This class contains the data a uses is receiving at the "/result" endpoint.
    """

    class Config:
        allow_mutation = False

    job: JobUser
    response: RRQueued


class QuantumCircuit(BaseModel):
    """
    A quantum circuit-type job that can run on a computing resource.
    """

    class Config:
        allow_mutation = False

    number_of_qubits: Annotated[int, Field(gt=0, title="Number Of Qubits")]
    quantum_circuit: Circuit
    repetitions: Annotated[int, Field(gt=0, title="Repetitions")]


class QuantumCircuits(BaseModel):
    """
    A collection of quantum circuits representing a single job.
    """

    class Config:
        allow_mutation = False

    circuits: Annotated[List[QuantumCircuit], Field(min_items=1, title="Circuits")]


class ResultResponse(BaseModel):
    class Config:
        allow_mutation = False

    __root__: Annotated[
        Union[
            JobResponseRRQueued,
            JobResponseRROngoing,
            JobResponseRRFinished,
            JobResponseRRError,
            JobResponseRRCancelled,
            UnknownJob,
        ],
        Field(
            examples={
                "cancelled": {
                    "description": (
                        "Job that has been cancelled by the user, before it could be processed by"
                        " the Quantum computer"
                    ),
                    "summary": "Cancelled Job",
                    "value": {
                        "job": {
                            "job_id": "ccaa39de-d0f3-4c8b-bdb1-4d74f0c2f450",
                            "job_type": "quantum_circuit",
                            "label": "Example computation",
                            "resource_id": "",
                            "workspace_id": "",
                        },
                        "response": {"status": "cancelled"},
                    },
                },
                "error": {
                    "description": (
                        "Job that created an error while being processed by the Quantum computer"
                    ),
                    "summary": "Failed Job",
                    "value": {
                        "job": {
                            "job_id": "ccaa39de-d0f3-4c8b-bdb1-4d74f0c2f450",
                            "job_type": "quantum_circuit",
                            "label": "Example computation",
                            "resource_id": "",
                            "workspace_id": "",
                        },
                        "response": {"message": "detailed error message", "status": "error"},
                    },
                },
                "finished": {
                    "description": (
                        "Job that has been successfully processed by a quantum computer or"
                        " simulator"
                    ),
                    "summary": "Finished Job",
                    "value": {
                        "job": {
                            "job_id": "ccaa39de-d0f3-4c8b-bdb1-4d74f0c2f450",
                            "job_type": "quantum_circuit",
                            "label": "Example computation",
                            "resource_id": "",
                            "workspace_id": "",
                        },
                        "response": {
                            "result": {0: [[1, 0], [1, 1], [0, 0], [1, 1], [1, 1]]},
                            "status": "finished",
                        },
                    },
                },
                "ongoing": {
                    "description": "Job that is currently being processed by the Quantum computer",
                    "summary": "Ongoing Job",
                    "value": {
                        "job": {
                            "job_id": "ccaa39de-d0f3-4c8b-bdb1-4d74f0c2f450",
                            "job_type": "quantum_circuit",
                            "label": "Example computation",
                            "resource_id": "",
                            "workspace_id": "",
                        },
                        "response": {"finished_count": 0, "status": "ongoing"},
                    },
                },
                "queued": {
                    "description": (
                        "Job waiting in the queue to be picked up by the Quantum computer"
                    ),
                    "summary": "Queued Job",
                    "value": {
                        "job": {
                            "job_id": "ccaa39de-d0f3-4c8b-bdb1-4d74f0c2f450",
                            "job_type": "quantum_circuit",
                            "label": "Example computation",
                            "resource_id": "",
                            "workspace_id": "",
                        },
                        "response": {"status": "queued"},
                    },
                },
                "unknown": {
                    "description": "The supplied job id could not be found",
                    "summary": "Unknown Job",
                    "value": {
                        "job_id": "3aa8b827-4ff0-4a36-b1a6-f9ff6dee59ce",
                        "message": "unknown job_id",
                    },
                },
            },
            title="ResultResponse",
        ),
    ]


class JobSubmission(BaseModel):
    """
    Abstract job that can run on a computing resource.
    """

    class Config:
        allow_mutation = False

    job_type: Annotated[Literal["quantum_circuit"], Field(title="Job Type")] = "quantum_circuit"
    label: Annotated[Optional[str], Field(title="Label")] = None
    payload: QuantumCircuits
